// CClosableTab class definitions
// by enkeyDEV(Ottavio84)

#include "stdafx.h"
#include "emule.h"
#include "ClosableTabCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// CClosableTabCtrl

IMPLEMENT_DYNAMIC(CClosableTabCtrl, CTabCtrl)
CClosableTabCtrl::CClosableTabCtrl()
{
	static const uint16 s_auIconResID[] = { IDI_CLOSE };

	m_pImgLst.Create(16, 16, ILC_COLOR8 | ILC_MASK, 0, 10);
	m_pImgLst.SetBkColor(CLR_NONE);
	FillImgLstWith16x16Icons(&m_pImgLst, s_auIconResID, ELEMENT_COUNT(s_auIconResID));
}

CClosableTabCtrl::~CClosableTabCtrl()
{
}

BEGIN_MESSAGE_MAP(CClosableTabCtrl, CTabCtrl)
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
END_MESSAGE_MAP()


// CClosableTabCtrl message handlers

void CClosableTabCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
	int ntabs = GetItemCount();
	CRect clsbutrect;
	for(int i = 0; i < ntabs; i++)
	{
		GetItemRect(i, clsbutrect);

		//clsbutrect.DeflateRect(clsbutrect.Width() - 5 - 9,  2 + 4,  4, clsbutrect.Height() - 9 - (2 + 4));	// mod. [FoRcHa]
		clsbutrect.DeflateRect(clsbutrect.Width() - 16, 0 , 0, 0);	// DonGato: better closing response
		//                     |                    |   |   |   |   |          |             |    |   |                                            
		// with of the button -|                    |   |   |   |   |          |             |    |   |
		// with of the cross -----------------------|   |   |   |   |          |             |    |   |
		// right border --------------------------------+-a |   | a-|          |             |    |   |               
		// regular border btw. top of cross and top of tab -+-b | b----------b | b---------b | b--|   |
		// empty pixels in the icon (top) ----------------------+------------c | c---------c | c------|
		// height of the button -----------------------------------------------|             |
		// height of the cross --------------------------------------------------------------|

		if(clsbutrect.PtInRect(point)) 
		{
			GetParent()->SendMessage(WM_CLOSETAB, (WPARAM) i);
			return; 
		}
	}
	
	CTabCtrl::OnLButtonDown(nFlags, point);
}

void CClosableTabCtrl::DrawItem(LPDRAWITEMSTRUCT lpDIS)	// Function modified by FoRcHa
{
	CRect rect = lpDIS->rcItem;
	IMAGEINFO info;
	bool bSelected = lpDIS->itemState & ODS_SELECTED;
	int nTabIndex = lpDIS->itemID;
	if(nTabIndex < 0) 
		return;

	TCHAR label[64];
	TC_ITEM tci;
	tci.mask = TCIF_TEXT|TCIF_IMAGE|TCIF_STATE;
	tci.pszText = label;
	tci.cchTextMax = 63;
	tci.dwStateMask = TCIS_HIGHLIGHTED;
	if(!GetItem(nTabIndex, &tci)) 
		return;

	CDC* pDC = CDC::FromHandle(lpDIS->hDC);
	if(!pDC) 
		return;

	LONG	lTop = rect.top;

	rect.top += ::GetSystemMetrics(SM_CYEDGE);

	// Draw image
	CImageList* pImageList = &m_pImgLst;
	if(pImageList && tci.iImage >= 0) 
	{
		// Get height of image so we 
		pImageList->GetImageInfo(0, &info);
		CRect ImageRect(info.rcImage);

		CRect rIcon = rect;
		rIcon.right -= bSelected ? 6 : 4;
		rIcon.top += 2;
		rIcon.left = rIcon.right - ImageRect.Width();
		rect.right = rIcon.left - 3;

		pImageList->Draw(pDC, 0, CPoint(rIcon.left, rIcon.top), ILD_TRANSPARENT);
	}

	pDC->SetBkMode(TRANSPARENT);
	pDC->SetTextColor((tci.dwState & TCIS_HIGHLIGHTED) ? RGB(196, 0, 0) : GetSysColor(COLOR_BTNTEXT));

	UINT	dwFormat = DT_SINGLELINE|DT_BOTTOM|DT_CENTER|DT_NOPREFIX|DT_NOCLIP;

	if(bSelected)
	{
		rect.top = lTop;
		dwFormat = DT_SINGLELINE|DT_VCENTER|DT_CENTER|DT_NOPREFIX|DT_NOCLIP;
	}
	pDC->DrawText(label, rect, dwFormat);
}

void CClosableTabCtrl::PreSubclassWindow()
{
	CTabCtrl::PreSubclassWindow();
	ModifyStyle(0, TCS_OWNERDRAWFIXED);
}

int CClosableTabCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CTabCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	ModifyStyle(0, TCS_OWNERDRAWFIXED);
	return 0;
}
