//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#pragma once
#include "SafeFile.h"
#include "types.h"
#include "packets.h"
#include "otherstructs.h"
#include <afxcoll.h>
#include <afxtempl.h>
#include <afxcmn.h>


// Server TCP flags
#define	SRV_TCPFLG_COMPRESSION		0x00000001
#define SRV_TCPFLG_AUXPORT			0x00000004

// Server UDP flags
#define	SRV_UDPFLG_EXT_GETSOURCES	0x00000001
#define	SRV_UDPFLG_EXT_GETFILES		0x00000002

struct IPRange_Struct2;

class CServer
{
public:
	enum EnumServerPriority
	{
		SERVERPRIORITY_LOW		= 0,
		SERVERPRIORITY_NORMAL	= 1,
		SERVERPRIORITY_HIGH		= 2,
		SERVERPRIORITY_INVALID, // this one invalid , should always be last
	};
	CServer(ServerMet_Struct* in_data);
	CServer(uint16 in_port, const CString& i_addr);
	CServer(const CServer& pOld);
	~CServer();
	void	AddTag(CServerTag* in_tag)						{m_tagList.AddTail(in_tag);}
	CString	GetListName() const								{return m_strListName;}
	CString	GetFullIP() const								{return m_strFullIP;}
	CString	GetAddress() const;
	uint16	GetPort() const									{return m_uPort;}
	void	SetPort(uint16 uPort)							{m_uPort = uPort;}
	uint16	GetAuxPort() const								{return m_uAuxPort;}
	void	SetAuxPort(uint16 uAuxPort)							{m_uAuxPort = uAuxPort;}
	bool	HasPublicAddress() const;
	bool	HasSameAddress(const CServer& server, bool bCheckPort = true) const;
	void	FillWindowTags(CTreeCtrl* wnd,HTREEITEM rootitem);
	bool	AddTagFromFile(CSafeFile& servermet);
	void	SetListName(const CString& newname) 			{ m_strListName = newname; }
	void	SetDescription(const CString& newname) 			{ description = newname; }
	uint32	GetIP()	const		{return m_dwIP;}
	uint32	GetFiles() const								{return m_dwFiles;} 
	uint32	GetNumUsers() const								{return m_dwNumUsers;} 
	CString	GetDescription() const							{return description;} 
	uint32	GetPing() const									{return m_dwPingTime;} 
	EnumServerPriority	GetPreferences() const				{return m_ePreferences;} 
	uint32	GetMaxUsers() const								{return m_dwMaxUsers;}
	void	SetMaxUsers(uint32 in_maxusers) 				{m_dwMaxUsers = in_maxusers;}
	void	SetUserCount(uint32 in_users)					{m_dwNumUsers = in_users;}
	void	SetFileCount(uint32 in_files)					{m_dwFiles = in_files;}
	void	ResetFailedCount()								{m_dwFailedCount = 0;}
	void	AddFailedCount()								{m_dwFailedCount++;}
	uint32	GetFailedCount() const							{return m_dwFailedCount;}
	void	SetID(uint32 newip);
	CString	GetDynIP() const								{return m_strDynIP;}
	bool	HasDynIP() const								{return !m_strDynIP.IsEmpty();}
	void	SetDynIP(const CString& newdynip)				{m_strDynIP = newdynip; }
	uint32	GetLastPingedTime()								{return lastpingedtime;}
	void	SetLastPingedTime(uint32 in_lastpingedtime)		{lastpingedtime = in_lastpingedtime;}
	uint32	GetLastPinged() const							{return lastpinged;}
	void	SetLastPinged(uint32 in_lastpinged)				{lastpinged = in_lastpinged;}
	byte	GetLastDescPingedCount()						{return lastdescpingedcount;}
	void	SetLastDescPingedCount(bool reset)				{if(reset){lastdescpingedcount=0;}else{lastdescpingedcount++;}}
	void	SetPing(uint32 in_ping)							{m_dwPingTime = in_ping;}
	void	SetPreference(EnumServerPriority in_preferences) {m_ePreferences = in_preferences;}
	void	SetIsStaticMember(bool bIsStatic)				{m_bIsStaticServerMember=bIsStatic;}
	bool	IsStaticMember() const							{return m_bIsStaticServerMember;}
	uint32	GetChallenge() const							{return challenge;}
	void	SetChallenge(uint32 in_challenge)				{challenge = in_challenge;}
	uint32	GetSoftMaxFiles() const							{return m_dwSoftMaxFiles;}
	void	SetSoftMaxFiles(uint32 in_dwNumSoftFiles)		{m_dwSoftMaxFiles = in_dwNumSoftFiles;}
	uint32	GetHardMaxFiles() const							{return m_dwHardMaxFiles;}
	void	SetHardMaxFiles(uint32 in_dwNumHardFiles)		{m_dwHardMaxFiles = in_dwNumHardFiles;}
	const	CString& GetVersion() const						{return m_strVersion;}
	void	SetVersion(const CString &strVersion)			{m_strVersion = strVersion;}

	uint32	ed2k2eMule(uint32 srvPref);	// Added by Tarod [thx Hein] - Convert eDonkey pref values to eMule pref values
	uint32	eMule2ed2k(uint32 srvPref);	// Added by Tarod [thx Hein] - Convert eMule pref values to eDonkey pref values

	void	SetTCPFlags(uint32 dwFlags)		{m_dwTCPFlags = dwFlags;}
	uint32	GetTCPFlags() const				{return m_dwTCPFlags;}
	void	SetUDPFlags(uint32 dwFlags)		{m_dwUDPFlags = dwFlags;}
	uint32	GetUDPFlags() const				{return m_dwUDPFlags;}

	HICON	GetServerInfo4Tooltips(CString &strInfo);
	CString GetUsersInfo4Tooltips();

	CString		GetCountryName() const;
	void		ResetIP2Country();

private:
	uint32		challenge;
	uint32		lastpinged;		//This is to get the ping delay. relative time (ms)
	uint32		lastpingedtime; //This is to decided when we retry the ping. absolute time (seconds) also saved in server.met
	byte		lastdescpingedcount;
	uint32		m_dwFiles;
	uint32		m_dwNumUsers;
	uint32		m_dwMaxUsers;
	uint32		m_dwSoftMaxFiles;
	uint32		m_dwHardMaxFiles;
	EnumServerPriority		m_ePreferences;
	uint32		m_dwPingTime;
	CString		description;
	CString		m_strListName;
	CString		m_strDynIP;
	uint32		m_dwTagCount;
	CString		m_strFullIP;
	uint32		m_dwIP;
	uint16		m_uPort;
	uint16		m_uAuxPort;
	uint32		m_dwFailedCount; 
	CTypedPtrList<CPtrList, CServerTag*>	m_tagList;
	bool		m_bIsStaticServerMember;
	CString		m_strVersion;

	uint32		m_dwTCPFlags;
	uint32		m_dwUDPFlags;

	uint16		m_uServerCountryIdx;
};